/***************************************************************************
                          ToyMem.h  -  description
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/

#ifndef __TOYMEM_H__
#define __TOYMEM_H__

#ifdef __cplusplus
  extern "C" {
#endif/*__cplusplus*/

#define MAX_PROCESSES 10
#define MIN_MEMSIZE   1000

typedef struct ToyMM
{
  struct Process *processList[MAX_PROCESSES]; /*  Processes                 */
  union ToyMem   *mem;                        /*  Memory elements           */
  int             memSize;                    /*  # of memory elements      */
  struct SegList *memFree;                    /*  Linked list of free space */

} ToyMM;

/*  Defines to simplify text output */
#define ERR_PREFIX  "******* ERROR: " /*  Error template  */
#define LINE_RULE   "+--------------------------------------------------\n"
#define LINE_PREFIX "| "  /*  Normal line prefix  */
#define LINE_NUMBER_FORMAT "%4d"  /*  The format for the printing of line #s  */

typedef enum ErrMessage_t
{
  ERR_UNKNOWN       =-1
, ERR_INVALIDINPUT  = 0
, ERR_PROCESSEXISTS = 1
, ERR_NOFILE
, ERR_INVALIDFILE
, ERR_DATAOVERFLOW
, ERR_INVALIDINSTRUCTION
, ERR_INVALIDDATA
, ERR_INSUFFICIENTMEM
, ERR_INVALIDADDRESS
, ERR_NOPROCESS
, ERR_NOPRIVILEGE
, ERR_PARSEEND
, ERR_NONE
} ErrMessage_t;
/*  List of error messages to match ErrMessage_t  */
const char *ErrMessages[];

/*  Creates a new Toy Memory Manager  */
ToyMM *TMM_Create(int memsize);

/*  Frees a Toy Memory Manager  */
ErrMessage_t TMM_Free(ToyMM *tmm);
ErrMessage_t ToyMM_Load(ToyMM *tm,int process,const char *filename,int addr);
ErrMessage_t TMM_PrintProcessTable(ToyMM *tm, int process);
ErrMessage_t TMM_PrintProcessMemory(ToyMM *tm, int process);
ErrMessage_t TMM_StoreInteger(ToyMM *tm,int process,int addr,int val);
ErrMessage_t TMM_StoreString(ToyMM *tm, int process, int addr, const char *str);
ErrMessage_t TMM_TerminateProcess(ToyMM *tm, int process);
ErrMessage_t TMM_PrintAllMemory(ToyMM *tm);

#ifdef __cplusplus
  }
#endif/*__cplusplus*/
 
#endif/*__TOYMEM_H__*/
