/***************************************************************************
                          ToyMemType.h  -  Defines the structures and
    main union that represent the Toy Memory Manager's main memory.
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/

#ifndef __TOYMEMTYPE_H__
#define __TOYMEMTYPE_H__

#ifdef __cplusplus
  extern "C"
#endif/*__cplusplus*/

#define MAX_STRLEN 20

/*
 *  Toy_MemType.h
 *  Defines the structures used for the type of memory
 *  the Toy Memory Manager handles;  a single slot in memory
 *  can be an integer, a string, or an instruction.
 *
 */


/*
 *  Types of instructions are merely run and halt.
 */
typedef enum InstructionType_e
{
  INSTRUCTION_ERROR = -1
, INSTRUCTION_RUN   = 0
, INSTRUCTION_HALT
} InstructionType_e;

/*
 * DataTypes:
 *  Integers - 32 bit unsigned #s.
 *  Strings - NULL-terminated ASCII string, maxlen 20 chars.
 *  Instruction - Enumerated type representing CPU instructions.
 */
typedef enum DataType_e
{
  DATA_ERROR    =-1
, DATA_MAGIC    = 0
, DATA_INTEGER  = 1
, DATA_STRING
, DATA_INSTRUCTION
} DataType_e;

typedef struct ToyMem_Integer
{
  DataType_e type;
  unsigned int value;
} ToyMem_Integer;

typedef struct ToyMem_String
{
  DataType_e type;
  char value[MAX_STRLEN+1];
} ToyMem_String;

typedef struct ToyMem_Instruction
{
  DataType_e type;
  InstructionType_e value;
} ToyMem_Instruction;

typedef union ToyMem
{
  DataType_e type;
  ToyMem_Instruction	instruction;
  ToyMem_String		string;
  ToyMem_Integer		integer;
} ToyMem;

#ifdef __cplusplus
  }
#endif/*__cplusplus*/

#endif/*__TOYMEMTYPE_H__*/
