/***************************************************************************
                          ToySegmentList.h
      A set of functions for creating, destroying, and
      manipulating doubly-linked lists of memory segments.

                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/
#ifndef __TOYSEGLIST_H__
#define __TOYSEGLIST_H__

#ifdef __cplusplus
  extern "C" {
#endif

typedef struct SegList
{
  struct ToySegment seg;
  struct SegList *prev;
  struct SegList *next;
} SegList;

/* Convenience macro.  Returns the index of the last
   byte in the segment.                               */
#define SegEnd(x) ((x)->pos + (x)->len)

/*  Makes a newly-allocated copy of a segment list */
SegList *TMM_CopySegList(SegList *list);

/*  Creates a new node for a segment list.  */
SegList *TMM_CreateSegList(int pos, int len, SegList *prev, SegList *next);
/*  Seeking functions.  */
SegList *TMM_SeekSegListHead(SegList *list);
SegList *TMM_SeekSegListEnd(SegList *list);

/*  Appends a node or nodes to the end of the list. */
SegList *TMM_AppendSegList(SegList *list, SegList *node);

/*  Frees a linked list of segments */
int TMM_FreeSegList(SegList *list);

/*  Compares two segments for sorting.  */
int TMM_CompSegList(const SegList *l1, const SegList *l2);

/*  Sorts the list in ascending order of segment address  */
SegList *TMM_SortSegList(SegList *list);

/*  Removes a memory area from the list */
SegList *TMM_RemoveAreaFromSegList(SegList *list, int pos, int len);

/*  Merges end-to-end segments  */
SegList *TMM_MergeSegList(SegList *list);

/*  Returns nonzero if the segments overlap, 0 if not.  */
int TMM_AreaIntersectsSeg(ToySegment *seg1, ToySegment *seg2);

/*  Adds another segment to the list, if the new segment
    doesn't overlap with any existing ones.               */
SegList *TMM_AddAreaToSegList(SegList *list, int pos, int len);

/* Prints the list of segments  */
void TMM_PrintSegList(SegList *list);

/*  Returns # of resultant segments.
    -1 means they don't intersect.
    0 means they cancelled out.
    1 or 2 means the result is in segOut. */
int TMM_SubtractSegs(ToySegment *seg1, ToySegment *seg2, ToySegment segOut[2]);

/*  Tries to find space for a segment int a list of
    free memory segments.                           */
int TMM_FindSegSpace(SegList *list, int n);

#ifdef __cplusplus
  }
#endif

#endif/*__TOYSEGLIST_H__*/
