/***************************************************************************
                          toyparse.c  -  Core code for the Toy Memory
    Manager's parsing engine.  Both ToyLoad.c and ToyParseCmd.c use it.
    
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>
#include "ToyMem.h"
#include "toyparse.h"
#include "toyparse_internal.h"



int DetermineSubType(int type,const char *str, const char **args[])
{
  int n=-1;
  if((args==NULL)||(type<0))  return(-1);

  /*  Ensure that 'type' is actually in the list  */
  for(n=0; (n<type)&&(args[n]!=NULL); n++);

  /*  Went past the end of the list */
  if(args[n]==NULL)  return(-1);

  return(FindInList(str,args[n]));
}                                                        

int FindInList(const char *str, const char *list[])
{
  int n;
  for(n=0; list[n]!=NULL; n++)
    if(CompareType(str,list[n])>=0) return(n);

  return(-1);
}


int DetermineType(const char *str, const char **table[])
{
  int i;

  for(i=0; table[i]!=NULL; i++)
  {
    if(table[i]!=NULL)
    if(FindInList(str,table[i])>=0)  return(i);
  }
  return(-1);
}

int CompareType(const char *str, const char *typestr)
{
  int error=0,strpos=0,typepos=0;
  if((str==NULL)||(typestr==NULL))  return -1;

  //Loop until entire string is parsed
  while(!error&&(str[strpos]!='\0')&&(typestr[typepos]!='\0'))
  {
    //Case sensitive string matching.
    while(typestr[typepos]==str[strpos])
    {
      if((typestr[typepos]=='\0')||(str[strpos]=='\0')||(typestr[typepos]=='\%'))
        break;

        typepos++;
        strpos++;
    }

    /* Three cases:   1) Both strings ended, which is OK.
                      2) One did, the other didn't.  Bad.
                      3) A variable argument was encountered in typestr.  */

    //Strings match
    if((typestr[typepos]=='\0')&&(str[strpos]=='\0'))
    {     return(0);    }
    else if((typestr[typepos]=='\0')||(str[strpos]=='\0'))
    {
      //Strings didn't match.
      return(-1);
    }
    else if(typestr[typepos]=='%')
      switch(toupper(typestr[typepos+1]))
      {
      case 'S':
        //Anything goes, so just return true
        return(0);
        break;

      case 'U':
        while(isdigit(str[strpos]))  strpos++;
        //Advance past "%u"
        typepos+=2;
        break;

      default:
        //End of string or invalid type
        return(-1);
      }
    else
    {
      return(-1);
    }
  }

  //Type matches
  if((str[strpos]=='\0')&&(typestr[typepos]=='\0'))
    return 0;
  else //types don't match
    return(-1);
}


//Allocate a new array if 'in' is NULL, or resize an existing one.
void *AllocateArray(void *in, int size, int num);

char **Explode(const char *strIn)
{
  char **strList=NULL;
  int pos=0,numStr=0;


  while(strIn[pos]!='\0')
  {
    //Ignore whitespace
    while(isspace(strIn[pos])) pos++;

    //If we haven't reached the end of the string, must be another word.
    if(strIn[pos]!='\0')
    {
      char buf[100];
      int bufPos=0;

      //If we find a comment, stop parsing the line
      if((strIn[pos]=='/')&&(strIn[pos+1]=='/'))
      {
        pos+=2;
        while(strIn[pos]!='\0') pos++;
        continue;
      }
      strList=(char **)AllocateArray(strList,sizeof(char *),numStr+1);
      numStr++;
      while(!isspace(strIn[pos])&&(strIn[pos]!='\0')&&(bufPos<(100-1)))
      {
        //Add letters to the end of our string
        buf[bufPos++]=strIn[pos++];
      }

      buf[bufPos++]='\0'; //NULL terminator
      //Allocate memory for string
      strList[numStr-1]=(char *)malloc(bufPos);
      //Copy string into new memory
      strcpy(strList[numStr-1],buf);
    }
  }

  if(strList==NULL)
  {
    //Don't bother returning any mem if the list is empty
    return(NULL);
  }
  else
  {
    //Tack a NULL onto the end of the list so we can tell where it ends
    strList=(char **)AllocateArray(strList,sizeof(char *),numStr+1);
    strList[numStr]=NULL;
    return(strList);
  }
}

void *AllocateArray(void *in, int size, int num)
{
  if(in==NULL)  return(malloc(size*num));
  else          return(realloc(in,size*num));
}

void FreeStrList(char **strList)
{
  int n;
  if(strList==NULL) return;

  for(n=0; strList[n]!=NULL; n++)
  {
    free(strList[n]);
  }

  free(strList);
}
