/***************************************************************************
                          toyparse.h
    Functions used in the Toy Memory Manager String Parser.

    How the parser works:
      At the root of it all, is the command table.  It contains
    a list of pointers;  each pointer points to a list of strings.
    Both the table and each list in it are NULL-terminated.    
      The strings denote valid commands or valid command forms.
    For instance, say you have the root category of 'value'.  Being
    the first thing in the command table, it would be at index 0,
    and point to a list of strings, like { "#%d","\"%s",NULL };
      %d means accept any sequence of numbers in that spot, %s means
    accept any integer values in that spot.  So both "#100" and "\"Hello"
    would be accepted as valid commands.  "#100" would be subcommand
    0, while "\"Hello" would be subcommand 1.
        
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Tyler Montbriand
    student #            : 200200370
    class                : CS330
    email                : tsm@accesscomm.ca 
***************************************************************************/

#ifndef __TOYPARSE_H__
#define __TOYPARSE_H__

#ifdef __cplusplus
  extern "C"  {
#endif/*__cplusplus*/

/*  A macro to simplify testing bounds ranges.
    If value evaluates to true, then func is performed
    before 'return (-1)'.                               */
#define BAIL_IF(value,func,errval)  if(value) {      \
                                      func;          \
                                      return(errval);\
                                    }



/*  Determine the root type of a token  */
int DetermineType(const char *str, const char **table[]);

/*  Determine the sub-type from a known type  */
int DetermineSubType(int type,const char *str, const char **args[]);                                    

/* Convert a string into a list of words - array
   of pointers ending in NULL                     */
char **Explode(const char *strIn);

/* Free the list of words */
void FreeStrList(char **strList);


#ifdef __cplusplus
  }
#endif/*__cplusplus*/

#endif/*__TOYPARSE_H__*/
